; GNUCHCP.ASM (v4) -- smaller NASM rewrite of GNUCHCP.PAS
;
; Saturday, June 8, 2013

cpu 8086

%define DOS int 21h
%define BIOS int 10h

CR equ 13
LF equ 10

FONTWIDTH equ 8
FONTHEIGHT equ 16
FONTFIRSTCHAR equ 0
FONTNUMCHARS equ 256
FONTSIZE equ FONTHEIGHT * FONTNUMCHARS

EGACOLOR equ 4
EGAMONO equ 5
PROGFX equ 6
VGAMONO equ 7
VGACOLOR equ 8

ARGC equ 80h
ARGV equ 81h

PRINTSTR equ 9
GETVIDEO equ 1Ah
OPENREAD equ 3D00h
READFILE equ 3Fh
CLOSEFILE equ 3Eh
EXITPROG equ 4Ch
LFNTOSFN equ 7160h
SUBSTED equ 80h
ROM8X16CHARSET equ 1104h
USERCHARTAB equ 1110h

section .text

org 100h

isEGAVGA:
  mov ah,GETVIDEO
  xor al,al
  BIOS
  cmp al,GETVIDEO
  jnz ExitError2
  cmp bl,PROGFX
  jz ExitError2
  cmp bl,EGACOLOR
  jb ExitError2
  cmp bl,VGACOLOR
  ja ExitError2
Cmdline:
  mov bx,word[ARGV+1]
  cmp byte[ARGV],CR
  jz HelpMe
IsASCII:
  cmp bx,'-a'
  jnz IsExtended
  mov word[fontinfo.numchars],127
  mov word[fontinfo.firstchar],0
  mov byte[arglen],3
IsExtended:
  cmp bx,'-e'
  jnz IsHelp
  mov word[fontinfo.numchars],128
  mov word[fontinfo.firstchar],128
  mov byte[arglen],3
IsHelp:
  cmp bx,'-h'
  jnz IsReset
HelpMe:
  mov ah,PRINTSTR
  mov dx,helpmsg
  DOS
  mov byte[myerr],255
  jmp short ExitError
IsReset:
  cmp word[ARGV+1],'-r'
  jnz GetFile
ResetFont:
  mov ax,ROM8X16CHARSET
  xor bx,bx
  BIOS
  jmp short ExitOkay
ExitError2:
  jmp short ExitError
GetFile:
  mov bl,byte[ARGC]
  xor bh,bh
MakeASCIIZ:
  mov byte[bx+ARGV],0
  mov bl,byte[arglen]
ShortenName:
  mov ax,LFNTOSFN
  mov ch,SUBSTED
  mov cl,1
  lea si,[bx+ARGV+1]
  mov di,font.name
  stc
  push si
  push di
  DOS
  pop ax
  pop dx
  jc OpenFile
  xchg ax,dx
OpenFile:
  mov ax,OPENREAD
  DOS
  jc ExitError
Read:
  xchg bx,ax
  mov ah,READFILE
  mov cx,FONTSIZE
  mov dx,font.data
  DOS
  jc ExitError
Close:
  mov ah,CLOSEFILE
  DOS
LoadFont:
  mov ax,USERCHARTAB
  mov bh,FONTHEIGHT
  xor bl,bl
  mov cx,word[fontinfo.numchars]
  mov dx,word[fontinfo.firstchar]
  mov bp,font.data
  BIOS

ExitOkay:
  mov byte[myerr],0
ExitError:
  mov ah,EXITPROG
  mov al,byte[myerr]
  DOS

section .data align=1
  myerr db 1
  arglen db 0

helpmsg:
  db 'GNUCHCP v4',CR,LF,CR,LF
  db '-a  only load first 127 chars (v1)',CR,LF
  db '-e  only load last  128 chars (v2)',CR,LF
  db '-r  reset font'
  db '$'

fontinfo:
  .numchars dw FONTNUMCHARS
  .firstchar dw FONTFIRSTCHAR

section .bss

font:
  .data resb FONTSIZE
  .name resb 128

; EOF
